#! /usr/bin/awk -f

##
## A Very Simple Backup Script
##
## Copyright (C) 2005, Begumisa Gerald
##
## Written By:
## 	Begumisa Gerald <gbegumisa@cfi.co.ug>
##
## With valuable input from:
##	Brian Candler
##
## THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
## WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
## MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
## ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
## WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
## ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
## OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
##

#
# The FULL backup script. Run this script, as a cron job, as often as you
# would like to have FULL backups of your system.  This could be monthly, or
# weekly etc...
#

BEGIN {
        ############# BEGIN CUSTOMIZABLE VARIABLES ################
        #
	# Set the variable below to the (text) file containing the list of
	# files (or directories) you wish to back up. This file should
	# contain one file (or directory) per line for this script to work
	file_list = "/backup/transfer.txt";

	# Set the variable below to 1 to run in debug mode.  Nothing is done
	# in this mode.
        debug = 0;

	# Set the rotate_level variable to the number of snapshots we would
	# like to have in the system at any one time. For example, if we are
	# doing monthly full backups and we would like to have backups of up
	# to 4 months back, we would set the variable below to 4 (simple,
	# huh? :-)
	rotate_level = 4;

	# Set the directory_name variable to the name which you would like
	# the script to save the backups.  It is to this name that will be
	# appended a number representing how old the files in that directory
	# are with 0 representing the newest (current) set of files. 
	# IMPORTANT!!! Remember to ensure this is the same name you use in
	# the incremental backup script!
	directory_name = "/tmp/week";

	# Set the variable below to the name of the server to which we will
	# be sending the backups.  Remember to set up SSH between the two
	# machines with public key authentication for automatic logins
        host     = "192.168.1.1";

	# The user who will be logging into the above host
	user     = "beg_g";

	# Set the variable below to the email addresses of the people
	# responsible for this backup system - this should usually be
	# yourself
	maint    = "beg_g@pop.eahd.or.ug";

	# Set the variable below to the name of the machine which is running
	# this script
	mother_host = "important-server.co.ug";

	# Set the variable below to restrict the script to a specific day of
	# the week - valid values are 1-7 where 1 is Monday.  To turn off
	# this restriction, leave the variable below as zero.
	day_of_week = 0;

	# Set the variable below to the location of the move command
	mv_location = "/bin/mv";

	# Set the variable below to the location of the rm command
	rm_location = "/bin/rm";

	# Set pre_compress to 1 if you wish the compression to be
	# done on the 'mother_host'.  You need to be aware that while
	# this will save on bandwidth (you will be transmitting an already
	# compressed stream), this will put a tremendous load on the
	# 'mother_host' (the load of archiving and compressing)
	pre_compress = 0;

	# To prevent the rotation of logs, set the variable below to
	# 1.  This is overridden by the command line argument --norotate,
	# if this argument is present.  A typical scenario where this
	# argument would make sense is when you do not need snapshots
	# of previous backups to be kept.
	no_rotate = 0;
        #
        ################ END CUSTOMIZABLE VARIABLES #############

	for(arg_c in ARGV)
		if(ARGV[arg_c] ~ /--norotate/)
			no_rotate = 1;

	# Generate directory names
	for(n_r = 0; n_r < rotate_level; n_r++)
		b_dir[n_r] = directory_name n_r;

	filename = b_dir[0] "/" strftime(mother_host "-full-%d-%m-%Y.tar.gz");

	if(!debug)
		if(day_of_week > 0)
			if(strftime("%u") != day_of_week)
				exit;

        # Create the directory system
	r_cmds = rm_location " -rf " b_dir[rotate_level-1];
	for(n_r = rotate_level-2; n_r >= 0; --n_r)
		r_cmds = r_cmds " ; " mv_location " " b_dir[n_r] " " b_dir[n_r+1];

	r_cmds = r_cmds " ; mkdir " b_dir[0];
        cmd = "ssh " user "@" host " \"" r_cmds "\"";

        if(debug) {
                rv = 254;
                print cmd;
        } else {
		if(!no_rotate)
                	rv = system(cmd);
        }

        if(rv) {
                if(rv == 255) {
			email_subject = "Full backup of " mother_host " FAILED";
			email_body = "The backup of " mother_host " to " host " failed because\n"\
                                     "of one or more wrong SSH settings which resulted in a\n"\
				     "login failure.\n\nPlease check your SSH settings.\n";
			system("echo \"" email_body "\" | mail -s \"" email_subject "\" " maint);

			if(!debug)
				exit;
                } else if(rv) {
			email_subject = "Full backup of " mother_host " FAILED";
			email_body = "The backup of " mother_host " to " host " failed because\n"\
				     "one or more remote ssh commands failed with status "rv".\n";
			system("echo \"" email_body "\" | mail -s \"" email_subject "\" " maint);
			if(!debug)
				exit;
		}
	}

	if(pre_compress == 1)
		cmd = "tar -Pcf - -T " file_list " | gzip -9c | ssh " user "@" host " \"dd of=" filename "\"";
	else
		cmd = "tar -Pcf - -T " file_list " | ssh " user "@" host " \"gzip -9c > " filename "\"";

	if(debug) {
		rv = 225;
		print(cmd);
        } else {
                system(cmd);
	}

	if(rv == 255) {
		email_subject = "Full backup of " mother_host " FAILED";
		email_body = "The backup of " mother_host " to " host " failed because\n"\
			     "of one or more wrong SSH settings which resulted in a\n"\
			     "login failure.\n\nPlease check your SSH settings.\n";
		system("echo \"" email_body "\" | mail -s \"" email_subject "\" " maint);
		exit;
	} else if(rv) {
		email_subject = "Full backup of " mother_host " FAILED";
		email_body = "The backup of " mother_host " to " host " failed because\n"\
			     "one or more remote ssh commands failed with status "rv".\n";
		system("echo \"" email_body "\" | mail -s \"" email_subject "\" " maint);
		exit;
	}

	email_subject = "Successful FULL backup of " mother_host;
	email_body = mother_host " has been successfully backed up, in full,\n"\
		     "to " host;

	system("echo \"" email_body "\" | mail -s \"" email_subject "\" " maint);
	exit;
}
