#! /usr/bin/awk -f

##
## A Very Simple Backup Script
##
## Copyright (C) 2005, Begumisa Gerald
##
## Written By:
##      Begumisa Gerald <gbegumisa@cfi.co.ug>
##
## With valuable input from:
##      Brian Candler
##
## THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
## WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
## MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
## ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
## WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
## ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
## OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
##

#
# The INCREMENTAL backup script.  Run this as often as you would like
# incremental backups to be taken - this could be hourly / daily / weekly
# etc...
#

BEGIN {
	############# BEGIN CUSTOMIZABLE VARIABLES ################
	#
        # Set the variable below to the (text) file containing the list of
        # files (or directories) you wish to back up. This file should
        # contain one file (or directory) per line for this system to work
        file_list = "/backup/transfer.txt";

	# Set the variable below to specify what time threshold to use. 
	# This will be directly fed to the "mtime" argument of the find
	# command (see the man page of find for specific information). 
	# Basically specify a digit as well as a unit valid units are 's'
	# for second(s), 'm' for minute(s), 'h' for hour(s), 'd' for day(s),
	# 'w' for week(s).  For example, if you specify '1d', all files
	# which were modified in the last one day from the above list will
	# be backed up.  Specifying 30m means all files which were modified
	# in the last 30 minutes will be backed up.
	#
	# Note that the "d" may not be supported by some implementations
	# of find. In such cases, simply remove the "d"
	time_threshold = "1d";

	# The name of the directory to which backups will be sent. Remember
	# that this should be the same as the one in the full backup script.
	directory_name = "/tmp/week";

        # Set the variable below to 1 to run in debug mode.  Nothing is done
        # in this mode.
	debug    = 0;

        # Set the variable below to the name of the server to which we will
        # be sending the backups.  Remember to set up SSH between the two
        # machines with public key authentication for automatic logins
        host     = "192.168.1.251";

        # Set the variable below to the name of the machine which is running
        # this script
	mother_host = "important-server.co.ug";

        # The user who will be logging into the above host
        user     = "beg_g";

        # Set the variable below to the email addresses of the people
        # responsible for this backup system - this should usually
        # be yourself
	maint	 = "admin1@host.com,admin2@host.com";

	# Set the variable below to 1 to use cpio, in the event that your
	# tar command doesn't support the -T option
	use_cpio = 0;

        # Set the variable below to the location of the rm command
        rm_location = "/bin/rm";

        # Set pre_compress to 1 if you wish the compression to be
        # done on the 'mother_host'.  You need to be aware that while
        # this will save on bandwidth (you will be transmitting an already
        # compressed stream), this will put a tremendous load on the
        # 'mother_host' (the load of archiving and compressing)
        pre_compress = 0;
	#
	################ END CUSTOMIZABLE VARIABLES #############


	filename = directory_name "0/" strftime(mother_host "-inc-week%U-%a-%d-%m-%Y-%H-%M-%S.tar.gz");
	i        = 0;
	daysold = time_threshold;
	bkfiles  = 0;

	if(!debug)
        	if(strftime("%u") == 7)
                	exit;

        while((("cat " file_list) | getline) > 0) {
                if($0 ~ /^#/) {
                        continue;
                }
		files[i++] = $0;
        }

	close("cat " file_list);

	temp_list_file_name = "/tmp/" strftime("asbc-temp-list-%Y%d%m%H%M%S-listing");
	listcmd = "cat >> " temp_list_file_name;

	for(file in files) {
		i = 0;
		l = 0;
		n = 0;
		cmd = "find " files[file] " -type f -mtime -" daysold;
		if(debug)
			print cmd;
		while((cmd | getline) > 0) {
			print $0 | listcmd;
			bkfiles = 1;
		}
		close(cmd);
	}

	if(!bkfiles) {
		if(debug)
			print "Ok. No files.\n";

		exit;
	}

	if(use_cpio) {
		if(pre_compress == 1)
			cmd = "cat " temp_list_file_name " | cpio --quiet -o -H ustar | gzip -9c | ssh " user "@" host " \"dd of=" filename"\"";
		else
			cmd = "cat " temp_list_file_name " | cpio --quiet -o -H ustar | ssh " user "@" host " \"gzip -9c >" filename"\"";
	} else {
		if(pre_compress == 1)
			cmd = "tar -Pcf - -T " temp_list_file_name " | gzip -9c | ssh " user "@" host " \"dd of=" filename"\"";
		else
			cmd = "tar -Pcf - -T " temp_list_file_name " | ssh " user "@" host " \"gzip -9c >" filename"\"";
	}

	if(debug) {
		rv = 1;
		print "";
		print cmd;
	} else {
		rv = system(cmd);
	}

	system(rm_location " -f " temp_list_file_name);

	if(rv == 255) {
		email_subject = "Incremental backup of " mother_host " FAILED";
		email_body = "The backup of " mother_host " to " host " failed because\n"\
			     "of one or more wrong SSH settings which resulted in a\n"\
			     "login failure.\n\nPlease check your SSH settings.\n";
		system("echo \"" email_body "\" | mail -s \"" email_subject "\" " maint);
		exit;
	} else if(rv) {
		email_subject = "Incremental backup of " mother_host " FAILED";
		email_body = "The backup of " mother_host " to " host " failed because\n"\
			     "one or more remote ssh commands failed with status "rv".\n";
		system("echo \"" email_body "\" | mail -s \"" email_subject "\" " maint);
		exit;
	}

	email_subject = "Successful Incremental backup of " mother_host;
	email_body = mother_host " has been successfully backed up to " host;

	system("echo \"" email_body "\" | mail -s \"" email_subject "\" " maint);
	exit;
}
